/*-------------------------------------------------------------------*/
/* CBPART. C      Juha Kivijrvi                                     */
/*                                                                   */
/* - Converts codebooks to partitionings and vice versa.             */
/*                                                                   */
/*-------------------------------------------------------------------*/

#define ProgName       "CBPART" 
#define VersionNumber  "V.0.03"
#define LastUpdated    "12.11.96"

/* ----------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <values.h>


#define  FACTFILE  "cbpart.fac"

#include "parametr.c"
#include "cb.h"
#include "file.h"
#include "memctrl.h"
#include "owntypes.h"

typedef enum { CBMODE, PAMODE } WORKINGMODE;


/*============================  P R I N T  ===============================*/


void PrintInfo(void)
{
  printf("\n%s %s %s\n", ProgName, VersionNumber, LastUpdated);
  printf("This program converts codebooks to partitionings and vice versa.\n");
  printf("Usage: %s [%coption] trainingset codebook     [partitioning]    or\n", ProgName, OPTION_SYMBOL);
  printf("       %s [%coption] trainingset partitioning [codebook]\n", ProgName, OPTION_SYMBOL);
  printf("\n  Options:\n");
  PrintOptions();
}


/*=========================  F I L E   N A M E S  ========================*/


WORKINGMODE AddExtensions(char* InName, char* OutName)
{
  CBFILETYPE InType;

  InType = DetermineCBFileTypeConsideringOrder(InName, 23);

  if (InType != NOTFOUND)
    {
    if ( strlen(OutName) < 1 )
      {
      PickFileName(InName, OutName);
      }
    CheckFileName(OutName, FormatName( InType==CBFILE? PAFILE : CBFILE ));
    }
  else
    {
    printf("ERROR: input file %s not found\n", InName);  
    exit(-1);
    }
    
  return (InType == CBFILE ? CBMODE : PAMODE);
}  


/*=========================  C O D E B O O K S  ==========================*/


void EmptyCodebook(CODEBOOK* CB)
{
  int i,j;
  
  for (i=0; i<BookSize(CB); i++)
    {
    for (j=0; j<VectorSize(CB); j++)
      VectorScalar(CB,i,j) = 0;
    VectorFreq(CB,i) = 0;
    }
}


/*===========================  M A I N  ==============================*/


int main(int argc, char* argv[])
{
  TRAININGSET   TS;
  CODEBOOK      CB;
  PARTITIONING  P;
  char          TSName[80];
  char          InName[80];
  char          OutName[80];
  WORKINGMODE   wm;

  ParameterInfo paraminfo[3] =  { { TSName,  FormatNameTS, 0, INFILE   }, 
                                  { InName,  "",           0, DONTCARE },
                                  { OutName, "",           1, OUTFILE  } };
                               
  ParseParameters( argc, argv, 3, paraminfo);
  wm = AddExtensions(InName, OutName);

  ReadTrainingSet(TSName, &TS);

  if ( wm == CBMODE )
    {
    ReadCodebook(InName, &CB);
    CreateNewPartitioning(&P, &TS, BookSize(&CB));
    GenerateOptimalPartitioning(&TS, &CB, &P);
    WritePartitioning(OutName, &P, &TS, Value(OverWrite));
    }
  else
    {
    ReadPartitioning(InName, &P, &TS);
    CreateNewCodebook(&CB, P.PartitionCount, &TS);
    EmptyCodebook(&CB);
    GenerateOptimalCodebook(&TS, &CB, &P);
    WriteCodebook(OutName, &CB, Value(OverWrite));
    }

  FreeCodebook(&TS);
  FreeCodebook(&CB);
  FreePartitioning(&P);

  checkmemory();
  
  return( 0 );
}

